//
// Created by robin on 1/9/18.
//

#include "dns/QDnsLabel.h"
#include <QDataStream>

namespace QDns
{

    QString QDnsLabel::parse(const QByteArray& data, quint32 pos, quint8 maxRecursion)
    {
        if (maxRecursion == 0)
        {
            return QString::null;
        }

        QString fullLabel = "";
        QDataStream dataStream(data);
        dataStream.skipRawData(pos);

        if (dataStream.device()->bytesAvailable() < 1)
        {
            return QString::null;
        }

        quint8 labelSize;
        dataStream >> labelSize;
        while (labelSize != 0)
        {
            if (labelSize >> 6 == 3)
            {
                if (dataStream.device()->bytesAvailable() < 1)
                {
                    return QString::null;
                }
                quint8 labelSize2;
                dataStream >> labelSize2;
                quint16 pointer = ((labelSize << 8) | labelSize2) - (quint16) 49152;
                auto labelEnd = parse(data, pointer, maxRecursion - (quint8) 1);
                if (labelEnd.isNull())
                {
                    return QString::null;
                }
                if (!fullLabel.isEmpty())
                {
                    fullLabel.append(".");
                }
                fullLabel.append(labelEnd);
                break;
            }
            else if (labelSize >> 6 == 0)
            {

                if (dataStream.device()->bytesAvailable() < labelSize + 1)
                {
                    return QString::null;
                }

                char label[labelSize];
                int read = dataStream.readRawData(label, labelSize);
                if (read != labelSize)
                {
                    return QString::null;
                }
                if (!fullLabel.isEmpty())
                {
                    fullLabel.append(".");
                }
                fullLabel.append(QByteArray(label, labelSize));
                dataStream >> labelSize;
            }
            else
            {
                return QString::null;
            }
        }
        return fullLabel;
    }
}